#!/usr/local/bin/perl
# domain_setup.cgi
# Create a new virtual domain

require './virtual-server-lib.pl';
$access{'create'} || &error($text{'form_ecannot'});
&require_apache();
&require_bind();
&require_useradmin();
&require_mail();
&require_mysql();
&require_postgres();
&require_acl();
&ReadParse();
&error_setup($text{'setup_err'});

# Validate inputs (check domain name to see if in use)
$in{'dom'} =~ /^[A-Za-z0-9\.\-]+$/ || &error($text{'setup_edomain'});
foreach $d (&list_domains()) {
	&error($text{'setup_edomain2'}) if (lc($d->{'dom'}) eq lc($in{'dom'}));
	}
$in{'email_def'} || $in{'email'} =~ /\S/ || &error($text{'setup_eemail'});
&check_ipaddress($in{'ip'}) || &error($text{'setup_eip'});
if ($in{'user_def'}) {
	$in{'dom'} =~ /^([^\.]+)/;
	$try1 = $user = $1;
	if (defined(getpwnam($1)) || $config{'longname'}) {
		$user = $in{'dom'};
		$try2 = $user;
		if (defined(getpwnam($user))) {
			&error(&text('setup_eauto', $try1, $try2));
			}
		}
	}
else {
	$user = $in{'user'};
	$user =~ /^[^\t :]+$/ || &error($text{'setup_euser2'});
	defined(getpwnam($user)) && &error($text{'setup_euser'});
	}
# XXX group should always be from domain!
defined(getgrnam($user)) && &error(&text('setup_egroup', $user));
$uconfig{'home_base'} || &error($text{'setup_ehomebase'});
if ($config{'home_quotas'}) {
        if ($in{'quota'} == -1) { $in{'quota'} = $in{'otherquota'} };
        if ($in{'uquota'} == -1) { $in{'uquota'} = $in{'otheruquota'} }; 
	$in{'quota'} =~ /^\d+$/ ||  &error($text{'setup_equota'});
	$in{'uquota'} =~ /^\d+$/ ||  &error($text{'setup_euquota'});
	}
($db = $user) =~ s/[\.\-]/_/g;
$in{'webalizer'} && !$in{'web'} && &error($text{'setup_ewebalizer'});

# Check for various clashes
$in{'dns'} && &check_bind_clash($in{'dom'}) &&
	&error($text{'setup_edns'});
$in{'web'} && &check_apache_clash($in{'dom'}) &&
	&error($text{'setup_eweb'});
$in{'mysql'} && &check_mysql_clash($db) &&
	&error(&text('setup_emysql', $db));
$in{'postgres'} && &check_postgres_clash($db) &&
	&error(&text('setup_epostgres', $db));
$in{'webmin'} && &check_webmin_clash($user) &&
	&error(&text('setup_ewebmin', $user));

# Make sure default Apache directives are sensible
!$in{'web'} || !($err = &check_apache_directives()) ||
	&error(&text('setup_edirectives', $err));

# Check if we have the virtual interface
&foreign_require("net", "net-lib.pl");
@boot = &net::boot_interfaces();
@active = &net::active_interfaces();
@allifaces = (@boot, @active);
($got) = grep { $_->{'address'} eq $in{'ip'} } @active;
if ($in{'virt'} && !$got) {
	($iface) = grep { $_->{'fullname'} eq $config{'iface'} } @allifaces;
	&error(&text('setup_eiface', $config{'iface'})) if (!$iface);
	&error(&text('setup_eiface2', $config{'iface'}))
		if (!$iface->{'address'});
	}

&header($text{'setup_title'}, "");
print "<hr>\n";

# Work out user and group IDs
@groups = &useradmin::list_groups();
%gtaken = map { $_->{'gid'}, 1 } @groups;
for($gid=$uconfig{'base_gid'}; $gtaken{$gid}; $gid++) { }
$ugid = $in{'group_def'} ? $gid : getgrnam($in{'group'});
@users = &useradmin::list_users();
%taken = map { $_->{'uid'}, 1 } @users;
for($uid=$uconfig{'base_uid'}; $taken{$uid}; $uid++) { }
$home = &useradmin::auto_home_dir($uconfig{'home_base'}, $user);

# Build up domain object
%dom = ( 'dom', $in{'dom'},
	 'user', $user,
	 'group', $user,
	 'quota', $in{'quota'},
	 'uid', $uid,
	 'gid', $gid,
	 'ugid', $ugid,
	 'owner', $in{'owner'},
	 'email', $in{'email_def'} ? undef : $in{'email'},
	 'mail', $in{'mail'},
	 'web', $in{'web'},
	 'webalizer', $in{'webalizer'},
	 'mysql', $in{'mysql'},
	 'postgres', $in{'postgres'},
	 'webmin', $in{'webmin'},
	 'name', $in{'name'},
	 'ip', $in{'ip'},
	 'home', $home,
	 'dns', $in{'dns'},
	 'pass', $in{'pass'},
	 'db', $db
	);

# Run the before command
&set_domain_envs(\%dom, "CREATE_DOMAIN");
$merr = &making_changes();
&error(&text('setup_emaking', "<tt>$merr</tt>")) if (defined($merr));

# Create the Unix group
print &text('setup_group', $user),"<br>\n";
&useradmin::lock_user_files();
%ginfo = ( 'group', $user,
	   'gid', $gid );
&useradmin::set_group_envs(\%ginfo, 'CREATE_GROUP')
	if (defined(&useradmin::set_group_envs));
&useradmin::making_changes();
&useradmin::create_group(\%ginfo);
&useradmin::made_changes();
if (!defined(getgrnam($user))) {
	print $text{'setup_ecrgroup'},"<p>\n";
	print "<hr>\n";
	&footer("", $text{'index_return'});
	exit;
	}
print $text{'setup_done'},"<p>\n";

# Create the Unix user
print &text('setup_user', $user),"<br>\n";
%uinfo = ( 'user', $user,
	   'uid', $uid,
	   'gid', $ugid,
	   'gid', $ugid,
	   'pass', &useradmin::encrypt_password($in{'pass'}),
	   'real', $in{'owner'},
	   'home', $home,
	   'shell', $config{'unix_shell'} );
&useradmin::set_user_envs(\%uinfo, 'CREATE_USER', $in{'pass'}, [ ])
	if (defined(&useradmin::set_user_envs));
&useradmin::making_changes();
&useradmin::create_user(\%uinfo);
&useradmin::made_changes();
&useradmin::unlock_user_files();
if (!defined(getpwnam($user))) {
	print $text{'setup_ecruser'},"<p>\n";
	print "<hr>\n";
	&footer("", $text{'index_return'});
	exit;
	}
print $text{'setup_done'},"<p>\n";

if ($config{'home_quotas'}) {
	# Set his quota
	print $text{'setup_quota'},"<br>\n";
	&quota::edit_user_quota($user, $config{'home_quotas'},
				$in{'uquota'}, 0, 0, 0);
	print $text{'setup_done'},"<p>\n";
	}

# Create his home directory, and copy files into it
print $text{'setup_home'},"<br>\n";
&system_logged("mkdir '$uinfo{'home'}'");
&system_logged("chmod '$uconfig{'homedir_perms'}' '$uinfo{'home'}'");
&system_logged("chown $uid:$ugid '$uinfo{'home'}'");
&copy_skel_files($config{'virtual_skel'}, \%uinfo, $user);

# Setup web directories
foreach $d ( [ $config{'html_dir'} || 'public_html', '755' ],
	     [ 'cgi-bin', '755' ],
	     [ 'logs', '755' ],
	     [ 'homes', '755' ] ) {
	&system_logged("mkdir '$uinfo{'home'}/$d->[0]' 2>/dev/null");
	&system_logged("chmod $d->[1] '$uinfo{'home'}/$d->[0]'");
	&system_logged("chown $uid:$ugid '$uinfo{'home'}/$d->[0]'");
	}
print $text{'setup_done'},"<p>\n";

if ($in{'mail'}) {
	# Add domain to the local domains list
	&setup_mail(\%dom);
	}

if ($in{'web'}) {
	# Create the website as www.domain and domain
	&setup_apache(\%dom);
	&restart_apache();
	}

if ($in{'webalizer'}) {
	# Set up webalizer module for site
	&setup_webalizer(\%dom);
	}

if ($in{'dns'}) {
	# Add to BIND config
	&setup_bind(\%dom);
	&restart_bind();
	}

if ($in{'mysql'}) {
	# Create MySQL database
	&setup_mysql(\%dom);
	}

if ($in{'postgres'}) {
	# Create PostgreSQL database
	&setup_postgres(\%dom);
	}

# Add virtual IP address, if needed
if ($in{'virt'} && !$got) {
	print $text{'setup_virt'},"<br>\n";
	local $vmax = 0;
	foreach $b (@allifaces) {
		$vmax = $b->{'virtual'}
			if ($b->{'name'} eq $iface->{'name'} &&
			    $b->{'virtual'} > $vmax);
		}
	$virt = { 'address' => $in{'ip'},
		  'netmask' =>
			$net::virtual_netmask || $iface->{'netmask'},
		  'broadcast' =>
			$net::virtual_netmask eq "255.255.255.255" ?
				$in{'ip'} : $iface->{'broadcast'},
		  'name' => $iface->{'name'},
		  'virtual' => $vmax+1,
		  'up' => 1 };
	$virt->{'fullname'} = $virt->{'name'}.":".$virt->{'virtual'};
	&net::save_interface($virt);
	&net::activate_interface($virt);
	$virtiface = $virt->{'fullname'};
	print $text{'setup_done'},"<p>\n";
	}

# Save domain details
print $text{'setup_save'},"<br>\n";
$dom{'iface'} = $virtiface;
&save_domain(\%dom);
print $text{'setup_done'},"<p>\n";

if ($in{'webmin'}) {
	# Create Webmin user to manage domain
	&setup_webmin(\%dom);
	&restart_webmin();
	}

# Notify the owner via email
&send_domain_email(\%dom);

# Add to this user's list of domains if needed
if (!&can_edit_domain(\%dom)) {
	$access{'domains'} = join(" ", split(/\s+/, $access{'domains'}),
				       $dom{'id'});
	&save_module_acl(\%access);
	}

# Run the after creation command
&made_changes();

print "<hr>\n";
&footer("", $text{'index_return'});


