#!/usr/local/bin/perl
# index.cgi
# Display a list of domains managed by this module

require './virtual-server-lib.pl';

if ($single_domain_mode) {
	# This user can edit just a single domain, so show only a menu of
	# icons for options he can use
	&header($text{'index_title2'}, "", undef, 1, 1);
	$d = &get_domain($access{'domains'});
	&domain_title($d);
	print "<hr>\n";
	@links = ( "list_users.cgi?dom=$access{'domains'}",
		   "list_aliases.cgi?dom=$access{'domains'}" );
	@titles = ( $text{'users_title'}, $text{'aliases_title'} );
	@icons = ( "images/users.gif", "images/aliases.gif" );
	&icons_table(\@links, \@titles, \@icons);
	print "<hr>\n";
	&footer("/", $text{'index'});
	exit;
	}

&header($text{'index_title'}, "", "index", 1, 1, 0,
	undef, undef, undef, &text('index_version', $module_info{'version'}));
print "<hr>\n";

$clink = "$gconfig{'webprefix'}/config.cgi?$module_name";

# Make sure networking is supported
if (!&foreign_check("net")) {
	&foreign_require("net", "net-lib.pl");
	if (!defined(&net::boot_interfaces)) {
		&index_error(&text('index_enet'));
		}
	}

if ($config{'web'}) {
	# Make sure Apache is installed
	&foreign_installed("apache", 1) == 2 ||
		&index_error(&text('index_eapache', "/apache/", $clink));
	}

if ($config{'dns'}) {
	# Make sure BIND is installed
	&foreign_installed("bind8", 1) == 2 ||
		&index_error(&text('index_ebind', "/bind8/", $clink));
	}

if ($config{'mysql'}) {
	# Make sure MySQL is installed
	&foreign_installed("mysql", 1) == 2 ||
		&index_error(&text('index_emysql', "/mysql/", $clink));
	}

if ($config{'postgres'}) {
	# Make sure PostgreSQL is installed
	&foreign_installed("postgresql", 1) == 2 ||
		&index_error(&text('index_epostgres', "/postgresql/", $clink));
	}

if ($config{'webalizer'}) {
	# Make sure Webalizer is installed
	&foreign_installed("webalizer", 1) == 2 ||
		&index_error(&text('index_ewebalizer', "/webalizer/", $clink));
	}

if ($config{'mail'}) {
	if ($config{'mail_system'} == 3) {
		# Work out which mail server we have
		if (&postfix_installed()) {
			$config{'mail_system'} = 0;
			}
		elsif (&sendmail_installed()) {
			$config{'mail_system'} = 1;
			}
		else {
			&index_error(&text('index_email'));
			}
		&write_file("$module_config_directory/config", \%config);
		}
	elsif ($config{'mail_system'} == 1) {
		# Make sure sendmail is installed
		if (!&sendmail_installed()) {
			&index_error(&text('index_esendmail', '/sendmail/',
					   "/config.cgi?$module_name"));
			}
		}
	elsif ($config{'mail_system'} == 0) {
		# Make sure postfix is installed
		if (!&postfix_installed()) {
			&index_error(&text('index_epostfix', '/postfix/',
					   "/config.cgi?$module_name"));
			}
		}
	}

if (!$config{'iface'}) {
	# Work out the network interface automatically
	&foreign_require("net", "net-lib.pl");
	foreach $a (&net::active_interfaces()) {
		if ($a->{'up'} && $a->{'virtual'} eq '' &&
		    &net::iface_type($a->{'name'}) =~ /ethernet/i) {
			$config{'iface'} = $a->{'fullname'};
			last;
			}
		}
	if (!$config{'iface'}) {
		&index_error(&text('index_eiface', "/config.cgi?$module_name"));
		}
	&write_file("$module_config_directory/config", \%config);
	}

$config{'home_quotas'} = '';
$config{'mail_quotas'} = '';
if ($config{'quotas'}) {
	# Make sure quotas are enabled, and work out where they are needed
	&require_useradmin();
	if (!$uconfig{'home_base'}) {
		print "<p><b>",&text('index_ehomebase'),"</b><p>\n";
		}
	else {
		&foreign_require("mount", "mount-lib.pl");
		($home_mtab, $home_fstab) =
			&mount_point($uconfig{'home_base'});
		($mail_mtab, $mail_fstab) =
			&mount_point(&mail_system_base());
		if (!$home_mtab) {
			print "<p><b>",&text('index_ehomemtab',
		    		"<tt>$uconfig{'home_base'}</tt>"),"</b><p>\n";
			}
		elsif (!$mail_mtab) {
			print "<p><b>",&text('index_emailmtab',
				"<tt>".&mail_system_base()."</tt>"),"</b><p>\n";
			}
		else {
			# Make sure quotas are enabled for the home dir
			if (!($home_mtab->[4] = &quota::quota_can(
				$home_mtab, $home_fstab)) ||
			    !&quota::quota_now($home_mtab, $home_fstab)) {
				print "<p><b>",&text('index_equota',
				    "<tt>$home_mtab->[0]</tt>",
				    "<tt>$uconfig{'home_base'}</tt>"),
				    "</b><p>\n";
				}
			else {
				# Everything looks ok!
				$config{'home_quotas'} = $home_mtab->[0];
				$config{'mail_quotas'} = $mail_mtab->[0];

				if ($home_mtab->[0] ne $mail_mtab->[0]) {
					# Show warning about different
					# filesystems for mail and home
					print "<p><b>",&text('index_ediff',
					    "<tt>$home_mtab->[0]</tt>",
					    "<tt>$uconfig{'home_base'}</tt>",
					    "<tt>$mail_mtab->[0]</tt>",
					    "<tt>".&mail_system_base()."</tt>"),
					    "</b><p>\n";
					}
				}
			}
		}
	}
&write_file("$module_config_directory/config", \%config);

# Setup the default templates
&ensure_template("domain-template");
&ensure_template("user-template");

# Display local users
if ($config{'localgroup'} && $access{'local'}) {
	print "<h3>$text{'index_header1'}</h3>\n";
	@lg = getgrnam($config{'localgroup'});
	@lusers = grep { $_->{'gid'} == $lg[2] } &list_all_users_quotas();
	if (@lusers) {
		print "<a href='edit_user.cgi?new=1&dom=0'>",
		      "$text{'index_uadd'}</a><br>\n";
		print "<table border width=100%>\n";
		print "<tr $tb> <td><b>$text{'users_name'}</b></td> ",
		      "<td><b>$text{'users_pop3'}</b></td> ",
		      "<td><b>$text{'users_real'}</b></td> ",
		      $config{'home_quotas'} ?
			  "<td><b>$text{'users_quota'}</b></td> " : "",
		      "<td><b>$text{'users_size'}</b></td> </tr>\n";
		foreach $u (@lusers) {
			print "<tr $cb>\n";
			print "<td><a href='edit_user.cgi?dom=0&",
			      "user=$u->{'user'}'>$u->{'user'}</a></td>\n";
			print "<td>$u->{'user'}</td>\n";
			print "<td>$u->{'real'}</td>\n";
			if ($config{'home_quotas'}) {
				print "<td>",int($u->{'squota'}),
				      " $text{'form_b'}</td>\n";
				}
			local ($sz) = &mail_file_size($u);
			if ($sz > 1000000) {
				$sz = int($sz / 1000000)." MB";
				}
			elsif ($sz > 1000) {
				$sz = int($sz / 1000)." kB";
				}
			elsif ($sz == 0) {
				$sz = $text{'users_empty'};
				}
			else {
				$sz = "$sz b";
				}
			print "<td>$sz</td>\n";
			print "</tr>\n";
			}
		print "</table>\n";
		}
	else {
		print "<b>$text{'index_nousers'}</b><p>\n";
		}
	print "<a href='edit_user.cgi?new=1&dom=0'>",
	      "$text{'index_uadd'}</a><p>\n";
	print "<hr>\n";
	}

# Display domains
print "<h3>$text{'index_header2'}</h3>\n";
@doms = grep { &can_edit_domain($_) } &list_domains();
if ($config{'display_max'} && @doms > $config{'display_max'}) {
	# Too many domains to display, so show a search form
	print "<b>$text{'index_toomany'}</b><p>\n";
	print "<form action=search.cgi>\n";
	print "<b>$text{'index_search'}</b>\n";
	print "<select name=field>\n";
	print "<option value=dom checked>$text{'index_search_dom'}\n";
	print "<option value=user>$text{'index_search_user'}\n";
	print "<option value=ip>$text{'index_search_ip'}\n";
	print "</select> <b>$text{'index_contains'}</b>\n";
	print "<input name=what size=30>\n";
	print "<input type=submit value='$text{'index_searchok'}'></form>\n";
	}
elsif (@doms) {
	# Show domains in a table
	&create_links();
	print "<br>\n";
	&domains_table(\@doms);
	}
else {
	print "<b>$text{'index_none'}</b><p>\n";
	}
&create_links();
print "<p>\n";

if ($access{'stop'}) {
	print "<hr>\n";
	print "<table width=100%>\n";

	if ($config{'web'}) {
		$apid = &get_apache_pid();
		if (!$apid || !kill(0, $apid)) {
			# Apache is down .. offer to start it
			&ss_form("start_apache.cgi", $text{'index_astart'},
						     $text{'index_astartdesc'});
			}
		else {
			# Apache is up .. offer to stop it
			&ss_form("stop_apache.cgi", $text{'index_astop'},
						    $text{'index_astopdesc'});
			}
		}

	if ($config{'dns'}) {
		$bpid = &get_bind_pid();
		if (!$bpid || !kill(0, $bpid)) {
			# BIND is down .. offer to start it
			&ss_form("start_bind.cgi", $text{'index_bstart'},
						   $text{'index_bstartdesc'});
			}
		else {
			# BIND is up .. offer to stop it
			&ss_form("stop_bind.cgi", $text{'index_bstop'},
						  $text{'index_bstopdesc'});
			}
		}

	if ($config{'mail'}) {
		if (!&is_mail_running()) {
			# Mail server is down .. offer to start it
			&ss_form("start_mail.cgi", $text{'index_mstart'},
						   $text{'index_mstartdesc'});
			}
		else {
			# Mail server is up .. offer to stop it
			&ss_form("stop_mail.cgi", $text{'index_mstop'},
						  $text{'index_mstopdesc'});
			}
		}

	print "</table>\n";
	}

print "<hr>\n";
&footer("/", $text{'index'});

sub index_error
{
print "<p>$_[0]<p>\n";
print "<hr>\n";
&footer("/", $text{'index'});
exit;
}

sub ss_form
{
print "<form action=$_[0]> <tr>\n";
if ($_[0] =~ /^start/) {
	print "<td><img src=images/down.gif></td>\n";
	}
else {
	print "<td><img src=images/up.gif></td>\n";
	}
print "<td><input type=submit value='$_[1]'></td>\n";
print "<td>$_[2]</td> </tr></form>\n";
}

# mount_point(dir)
# Returns both the mtab and fstab details for the parent mount for a directory
sub mount_point
{
local $dir = &resolve_links($_[0]);
local @mounts = &mount::list_mounts();
foreach $m (sort { length($b->[0]) <=> length($a->[0]) }
	         &mount::list_mounted()) {
	if ($dir eq $m->[0] || $m->[0] eq "/" ||
	    substr($dir, 0, length($m->[0])+1) eq "$m->[0]/") {
		local ($m2) = grep { $_->[0] eq $m->[0] } @mounts;
		if ($m2) {
			return ($m, $m2);
			}
		}
	}
&error("Failed to find mount point for $dir");
}

sub create_links
{
if ($access{'create'}) {
	print "<a href='domain_form.cgi'>$text{'index_add'}</a>&nbsp;\n";
	if ($access{'import'}) {
		print "<a href='import_form.cgi'>$text{'index_import'}</a>\n";
		}
	}
}

