#!/usr/local/bin/perl
# save_domain.cgi
# Update or delete a domain

require './virtual-server-lib.pl';
&require_apache();
&require_bind();
&require_useradmin();
&require_mail();
&ReadParse();
$d = &get_domain($in{'dom'});
$access{'edit'} && &can_edit_domain($d) || &error($text{'edit_ecannot'});
$oldd = { %$d };

# Validate inputs
&error_setup($text{'save_err'});
if ($config{'home_quotas'}) {
	if ($in{'quota'} eq -1) { $in{'quota'} = $in{'otherquota'} };
	if ($in{'uquota'} eq -1) { $in{'uquota'} = $in{'otheruquota'} };
	$in{'quota'} =~ /^\d+$/ || &error($text{'save_equota'});
	$in{'uquota'} =~ /^\d+$/ || &error($text{'save_euquota'});
	}
($d->{'db'} = $d->{'user'}) =~ s/[\.\-]/_/g if (!$d->{'db'});

# Check for various clashes
!$d->{'dns'} && $in{'dns'} && &check_bind_clash($d->{'dom'}) &&
	&error($text{'setup_edns'});
!$d->{'web'} && $in{'web'} && &check_apache_clash($d->{'dom'}) &&
	&error($text{'setup_eweb'});
!$d->{'mysql'} && $in{'mysql'} && &check_mysql_clash($d->{'db'}) &&
	&error(&text('setup_emysql', $d->{'db'}));
!$d->{'postgres'} && $in{'postgres'} &&
   &check_postgres_clash($d->{'db'}) &&
	&error(&text('setup_epostgres', $d->{'db'}));
!$d->{'webmin'} && $in{'webmin'} && &check_webmin_clash($d->{'user'}) &&
	&error(&text('setup_ewebmin', $d->{'user'}));

# Make sure default Apache directives are sensible
$d->{'web'} || !$in{'web'} || !($err = &check_apache_directives()) ||
	&error(&text('setup_edirectives', $err));

# Make the changes
&header($text{'save_title'}, "");
print "<hr>\n";

# Run the before command
&set_domain_envs($d, "MODIFY_DOMAIN");
$merr = &making_changes();
&error(&text('save_emaking', "<tt>$merr</tt>")) if (defined($merr));

# Change unix user details
local @allusers = &useradmin::list_users();
local ($uinfo) = grep { $_->{'user'} eq $d->{'user'}} @allusers;
if ($uinfo) {
	print $text{'save_user'},"<br>\n";
	$uinfo->{'real'} = $in{'owner'};
	if (!$in{'passwd_def'}) {
		$enc = &useradmin::encrypt_password($in{'passwd'});
		if ($d->{'disabled'}) {
			# Clear any saved passwords, as they should
			# be reset at this point
			$d->{'disabled_oldpass'} = $enc;
			$d->{'disabled_mysqlpass'} = undef;
			$d->{'disabled_postgrespass'} = undef;
			}
		else {
			$uinfo->{'pass'} = $enc;
			}
		$d->{'pass'} = $in{'passwd'};
		}
	&useradmin::set_user_envs($uinfo, 'MODIFY_USER',
				  $in{'passwd_def'} ? "" : $in{'passwd'})
		if (defined(&useradmin::set_user_envs));
	&useradmin::making_changes();
	&useradmin::modify_user($uinfo, $uinfo);
	&useradmin::made_changes();
	print $text{'setup_done'},"<p>\n";
	$d->{'owner'} = $in{'owner'};
	}

# Change Unix quota
if ($config{'home_quotas'}) {
	print $text{'save_quota'},"<br>\n";
	&quota::edit_user_quota($d->{'user'}, $config{'home_quotas'},
				$in{'uquota'}, 0, 0, 0);
	print $text{'setup_done'},"<p>\n";
	}

if (!$d->{'disabled'}) {
	if ($in{'web'} && !$d->{'web'}) {
		# Enable website
		&setup_apache($d);
		&restart_apache();
		}
	elsif (!$in{'web'} && $d->{'web'}) {
		# Disable website
		&delete_apache($d);
		&restart_apache();
		}
	elsif ($in{'web'}) {
		# Make any needed changes to the Apache config
		&modify_apache($d, $oldd) && &restart_apache();
		}
	$d->{'web'} = $in{'web'};

	if ($in{'dns'} && !$d->{'dns'}) {
		# Enable DNS zone
		&setup_bind($d);
		&restart_bind();
		}
	elsif (!$in{'dns'} && $d->{'dns'}) {
		# Disable DNS zone
		&delete_bind($d);
		&restart_bind();
		}
	elsif ($in{'dns'}) {
		# Make any needed changes to the DNS zone
		&modify_bind($d, $oldd) && &restart_bind();
		}
	$d->{'dns'} = $in{'dns'};

	if ($in{'mysql'} && !$d->{'mysql'}) {
		# Enable MySQL DB
		&setup_mysql($d);
		}
	elsif (!$in{'mysql'} && $d->{'mysql'}) {
		# Disable MySQL DB
		&delete_mysql($d);
		}
	elsif ($in{'mysql'}) {
		# Change MySQL settings
		&modify_mysql($d, $oldd);
		}
	$d->{'mysql'} = $in{'mysql'};

	if ($in{'postgres'} && !$d->{'postgres'}) {
		# Enable PostgreSQL DB
		&setup_postgres($d);
		}
	elsif (!$in{'postgres'} && $d->{'postgres'}) {
		# Disable PostgreSQL DB
		&delete_postgres($d);
		}
	elsif ($in{'postgres'}) {
		# Change PostgreSQL settings
		&modify_postgres($d, $oldd);
		}
	$d->{'postgres'} = $in{'postgres'};

	if ($in{'webalizer'} && !$d->{'webalizer'}) {
		# Create webalizer config
		&setup_webalizer($d);
		}
	elsif (!$in{'webalizer'} && $d->{'webalizer'}) {
		# Delete webalizer config
		&delete_webalizer($d);
		}
	elsif ($in{'webalizer'}) {
		&modify_webalizer($d, $oldd);
		}
	$d->{'webalizer'} = $in{'webalizer'};

	if ($in{'mail'} && !$d->{'mail'}) {
		# Enable mail for domain
		&setup_mail($d);
		}
	elsif (!$in{'mail'} && $d->{'mail'}) {
		# Disable mail for domain
		&delete_mail($d);
		}
	elsif ($in{'mail'}) {
		# Change mail for domain
		&modify_mail($d, $oldd);
		}
	$d->{'mail'} = $in{'mail'};

	if ($in{'webmin'} && !$d->{'webmin'}) {
		# Create webmin user
		&setup_webmin($d);
		&restart_webmin();
		}
	elsif (!$in{'webmin'} && $d->{'webmin'}) {
		# Delete webmin user
		&delete_webmin($d);
		&restart_webmin();
		}
	elsif ($in{'webmin'}) {
		&modify_webmin($d, $oldd) && &restart_webmin();
		}
	$d->{'webmin'} = $in{'webmin'};
	}

# Save new domain details
print $text{'save_domain'},"<br>\n";
$d->{'quota'} = $in{'quota'};
&save_domain($d);
print $text{'setup_done'},"<p>\n";

# Run the after command
&set_domain_envs($d, "MODIFY_DOMAIN");
&made_changes();

print "<hr>\n";
&footer("edit_domain.cgi?dom=$in{'dom'}", $text{'edit_return'},
	"", $text{'index_return'});

