#!/usr/local/bin/perl
# save_user.cgi
# Create, update or delete a user
# XXX make sure lusers still work

require './virtual-server-lib.pl';
&ReadParse();
$d = &get_domain($in{'dom'});
&can_edit_domain($d) || &error($text{'users_ecannot'});
@users = &list_domain_users($d);
@virts = &list_virtusers();
if (!$in{'new'}) {
	# Lookup user details
	($user) = grep { $_->{'user'} eq $in{'old'} } @users;
	%old = %$user;
	($virt) = grep { $_->{'from'} eq $in{'oldpop3'}."\@".$d->{'dom'} }
		       @virts;
	%oldvirt = %$virt;

	# Find old extra addresses
	foreach $v (@virts) {
		if (@{$v->{'to'}} == 1 &&
		    $v->{'to'}->[0] eq $in{'old'} &&
		    $v ne $virt) {
			push(@oldextra, $v);
			}
		}
	}
&error_setup($text{'user_err'});

@allusers = &useradmin::list_users();
%taken = map { $_->{'uid'}, 1 } @allusers;
%utaken = map { $_->{'user'}, 1 } @allusers;
($du) = grep { $_->{'user'} eq $d->{'user'} } @allusers;

if ($in{'delete'}) {
	# Get rid of his mail file
	&delete_mail_file($user);

	# Just delete this user
	&useradmin::set_user_envs($user, 'DELETE_USER')
		if (defined(&useradmin::set_user_envs));
	&useradmin::making_changes();
	&useradmin::delete_user($user);
	&useradmin::made_changes();

	# Delete the virtuser too
	if ($virt) {
		&delete_virtuser($virt);
		}

	# Delete another other virtusers that point to this pop3 user
	foreach $v (@oldextra) {
		&delete_virtuser($v);
		}

	# Delete home dir
	system("rm -rf '$user->{'home'}'")
	  if (-d $user->{'home'} && $user->{'home'} ne "/");
	}
else {
	# Verify inputs
	$in{'user'} =~ /^[^ \t:]+$/ || &error($text{'user_euser'});
	$in{'real'} =~ /^[^:]*$/ || &error($text{'user_ereal'});
	$user->{'real'} = $in{'real'};
	$user->{'pass'} = &useradmin::encrypt_password($in{'pass'})
		if (!$in{'pass_def'});
	if ($config{'home_quotas'}) {
                if ( $in{'quota'} eq -1 ) { $in{'quota'} = $in{'otherquota'} };
		$in{'quota'} =~ /^\d+$/ || &error($text{'user_equota'});
		}
	@extra = split(/\s+/, $in{'extra'});
	foreach $e (@extra) {
		$e =~ /^(\S+)\@(\S+)$/ || &error(&text('user_eextra1', $e));
		local ($eu, $ed) = ($1, $2);
		local $edom = &get_domain_by("dom", $ed);
		$edom && $edom->{'mail'} || &error(&text('user_eextra2', $ed));
		&can_edit_domain($edom) || &error(&text('user_eextra3', $ed));
		}

	# Find home
	local ($du) = grep { $_->{'user'} eq $d->{'user'} } @allusers;
	local $home = "$du->{'home'}/homes/$in{'user'}";

	# Create or update the user
	if ($in{'new'}) {
		# Set new user parameters
		$user->{'uid'} = $uconfig{'base_uid'};
		while($taken{$user->{'uid'}}) {
			$user->{'uid'}++;
			}
		$user->{'gid'} = $d->{'gid'};
		$user->{'shell'} = $in{'ftp'} ? $config{'ftp_shell'}
					      : $config{'shell'};
		$user->{'home'} = $home;
		if ($utaken{$in{'user'}} || $config{'append'}) {
			# Oh no .. name is in use! Add group name
			$user->{'user'} = &userdom_name($in{'user'}, $d);
			}
		else {
			$user->{'user'} = $in{'user'};
			}
		$user->{'passmode'} = 3;
		$user->{'plainpass'} = $in{'pass'};

		# Check for a clash in this domain
		if (&check_clash($in{'user'}, $d->{'dom'})) {
			&error($text{'user_eclash'});
			}

		# Check if any extras clash
		foreach $e (@extra) {
			$e =~ /^(\S+)\@(\S+)$/;
			if (&check_clash($1, $2)) {
				&error(&text('user_eextra4', $e));
				}
			}

		# Check if the name is too long
		if ($lerr = &too_long($user->{'user'})) {
			&error($lerr);
			}

		# Create the user
		&useradmin::set_user_envs($user, 'CREATE_USER', $in{'pass'}, [])
			if (defined(&useradmin::set_user_envs));
		&useradmin::making_changes();
		&useradmin::create_user($user);
		&useradmin::made_changes();

		# Need to create virtuser
		if ($in{'mailbox'}) {
			$virt->{'from'} = $in{'user'}."\@".$d->{'dom'};
			$virt->{'to'} = [ $user->{'user'} ];
			&create_virtuser($virt);
			}

		# Create any extra email addresses
		foreach $e (@extra) {
			$evirt = { 'from' => $e,
				   'to' => [ $user->{'user'} ] };
			&create_virtuser($evirt);
			}

		# Create his homedir
		system("mkdir -p '$home'");
		system("chown $user->{'uid'}:$user->{'gid'} '$home'");
		system("chmod 755 '$home'");

		# Copy files into homedir
		&copy_skel_files($config{'mail_skel'}, $user);

		# Send an email upon creation
		@erv = &send_user_email($d, $user);
		print STDERR "$erv[0] $erv[1]\n";
		}
	else {
		# Check if any extras clash
		%oldextra = map { $_->{'from'}, 1 } @oldextra;
		foreach $e (@extra) {
			$e =~ /^(\S+)\@(\S+)$/;
			if (!$oldextra{$e} && &check_clash($1, $2)) {
				&error(&text('user_eextra4', $e));
				}
			}

		# Update user parameters (handle rename and .group)
		if ($in{'user'} ne $in{'oldpop3'}) {
			# Has been renamed .. check for a username clash
			if ($utaken{$in{'user'}} || $config{'append'}) {
				# New name has to include group
				$user->{'user'} = &userdom_name($in{'user'},$d);
				}
			else {
				# Can rename without the dot
				$user->{'user'} = $in{'user'};
				}

			# Check if the name is too long
			if ($lerr = &too_long($user->{'user'})) {
				&error($lerr);
				}

			# Check for a virtuser clash too
			if (&check_clash($in{'user'}, $d->{'dom'})) {
				&error($text{'user_eclash'});
				}

			# Rename his mail file
			&rename_mail_file($user, $in{'oldpop3'});

			# Rename homedir
			if ($user->{'home'} ne $home && -d $user->{'home'}) {
				system("mv '$user->{'home'}' '$home'");
				$user->{'home'} = $home;
				}
			}

		# Update shell
		if (defined($in{'ftp'})) {
			$user->{'shell'} = $in{'ftp'} ? $config{'ftp_shell'}
						      : $config{'shell'};
			}

		# Modify the user
		&useradmin::set_user_envs($user, 'MODIFY_USER',
					  $in{'pass_def'} ? "" : $in{'pass'})
			if (defined(&useradmin::set_user_envs));
		&useradmin::making_changes();
		&useradmin::modify_user(\%old, $user);
		&useradmin::made_changes();

		if ($in{'mailbox'}) {
			# Modify or create virtuser
			$virt->{'from'} = $in{'user'}."\@".$d->{'dom'};
			$virt->{'to'} = [ $user->{'user'} ];
			if (%oldvirt) {
				&modify_virtuser(\%oldvirt, $virt);
				}
			else {
				&create_virtuser($virt);
				}
			}
		elsif ($virt && !$in{'mailbox'}) {
			# Delete virtuser (as he no longer has a mail address)
			&delete_virtuser($virt);
			}

		# Remove all extra addresses for the old name, then add
		# back those for the new name.
		foreach $v (@oldextra) {
			&delete_virtuser($v);
			}
		foreach $e (@extra) {
			$evirt = { 'from' => $e,
				   'to' => [ $user->{'user'} ] };
			&create_virtuser($evirt);
			}
		}

	if ($config{'home_quotas'}) {
		# Set his quotas
		&quota::edit_user_quota($user->{'user'},
					$config{'home_quotas'},
					$in{'quota'}, 0, 0, 0);
		}
	}
&redirect("list_users.cgi?dom=$in{'dom'}");

